/*
 * Copyright (C) 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once

#include <string>
namespace ai_engine::core_ai::embedding {
typedef std::string TaskID;

class EmbeddingTask {
public:
    enum class EmbeddingDataType {
        Text = 0,
        TextByImageModel,
        Image,
        ImageBase64
    };
    enum class TaskPriority { LOW = 0, MEDIUM, HIGH, CRITICAL };
    EmbeddingTask() = default;
    EmbeddingTask(const std::string& data, EmbeddingDataType type,
                  TaskPriority priority = TaskPriority::MEDIUM);
    std::string getData() const { return data_; }

    TaskID id() const { return taskId_; }
    EmbeddingDataType type() const { return taskType_; }
    TaskPriority priority() const { return priority_; }

    bool operator<(const EmbeddingTask& other) const;

private:
    std::string data_;
    EmbeddingDataType taskType_;
    TaskID taskId_;
    TaskPriority priority_;
};
}  // namespace ai_engine::core_ai::embedding
